/* TileRasterCache, a library to download tiles from several sources.
 *
 * Copyright (C) 2010 Prodevelop SL.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *  
 *   Prodevelop Integraci�n de Tecnolog�as SL
 *   Plaza don Juan de Vilarrasa , 14-5
 *   46000 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *   
 *   author: Alberto Romeu Carrasco (aromeu@prodevelop.es)
 */

package es.prodevelop.tilecache.renderer.wms;

import java.util.logging.Level;
import java.util.logging.Logger;

import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.tilecache.renderer.MapRenderer;
import es.prodevelop.tilecache.renderer.MapRendererFactory;

public class WMSMapRendererFactory extends MapRendererFactory {

	private final static Logger logger = Logger
			.getLogger(WMSMapRendererFactory.class.getName());

	/**
	 * Instantiates a MapRenderer. @see Layers and layers.txt
	 * 
	 * @param layerTitle
	 *            The name of the MapRenderer
	 * @param layerProps
	 *            The properties of the MapRenderer
	 * @return
	 */
	public MapRenderer getMapRenderer(String layerTitle, String[] layerProps) {
		MapRenderer renderer = super.getMapRenderer(layerTitle, layerProps);
		String offlineLayerName = null;
		Extent offExtent = null;
		if (renderer == null) {
			try {
				int index = layerProps[0].indexOf("[");
				
				
				if (index != -1) {
					try {
						String ex = layerProps[0].substring(index + 1,
								layerProps[0].length() - 1);
						if (ex != null && ex.compareTo("") != 0) {
							String[] parts = ex
									.split(MapRenderer.NAME_SEPARATOR);
							offExtent = Extent.parseString(parts[0], ":");
							offlineLayerName = parts[1];
						}
					} catch (Exception e) {

					}

				}

				int rendererType = Integer.valueOf(
						layerProps[0].substring(0, 1)).intValue();
				switch (rendererType) {
				case MapRenderer.WMS_RENDERER:
					logger.log(Level.FINE, "Found a WMS layer: " + layerTitle);
					renderer = (WMSRenderer) WMSRenderer.getWMSRenderer(
							layerProps, layerTitle);
					break;
				case MapRenderer.OS_RENDERER:
					logger.log(Level.FINE, "Found a OS layer: " + layerTitle);
					renderer = (OSRenderer) OSRenderer.getOSRenderer(
							layerProps, layerTitle);
					break;
				}

			} catch (Exception e) {
				logger.log(Level.SEVERE, "getMapRenderer: ", e);
				return renderer;
			}
		}
		if (renderer != null && offlineLayerName != null) {
			renderer.setOfflineLayerName(offlineLayerName);
			renderer.setOfflineExtent(offExtent);
		}
			
		return renderer;
	}

}
