/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2009 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2009.
 *   author Alberto Romeu aromeu@prodevelop.es 
 *   author Ruben Blanco rblanco@prodevelop.es 
 *   
 */

package es.prodevelop.tilecache.renderer.wms;

import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.gvsig.remoteclient.utils.Utilities;
import org.gvsig.remoteclient.wms.WMSStatus;

import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.gvsig.mini.projection.TileConversor;
import es.prodevelop.gvsig.mini.wms.WMSCancellable;
import es.prodevelop.tilecache.renderer.MapRenderer;
import es.prodevelop.tilecache.renderer.TMSRenderer;

/**
 * MapRenderer for WMS servers
 * 
 * @author aromeu
 * @author rblanco
 * 
 */
public class WMSRenderer extends TMSRenderer {

	private final static Logger logger = Logger.getLogger(TMSRenderer.class
			.getName());

	String[] layers;
	String info_format = "";
	String version = "";
	WMSCancellable neverCanceled = new WMSCancellable();
	public static final int DEFAULT_MAX_ZOOM_LEVEL = 20;

	WMSRenderer(String[] baseUrl, String name, String imageFilenameEnding,
			int zoomMax, int zoomMin, int tileSizePX, String[] layers,
			Extent bbox, String srs, String version, Extent offExtent) {
		super(baseUrl, name, imageFilenameEnding, zoomMax, zoomMin, tileSizePX,
				MapRenderer.WMS_RENDERER, bbox.getLefBottomCoordinate(), bbox,
				srs, null, offExtent);
		try {
			this.layers = layers;

			double width = getExtent().getWidth();
			double height = getExtent().getHeight();
			this.version = version;

			if (width > height)
				calculateResolutions(width, zoomMax);
			else
				calculateResolutions(height, zoomMax);
		} catch (Exception e) {
			logger.log(Level.SEVERE, "WMSRenderer: ", e);
		}
	}

	/**
	 * Instantiates a WMSRenderer. @see Layers and layers.txt
	 * 
	 * @param props
	 *            The properties of the WMSRenderer
	 * @param layerName
	 *            The name of the layer
	 * @return A WMSRenderer
	 */
	public static WMSRenderer getWMSRenderer(String[] props, String layerName) {
		try {

			TMSRenderer w = (TMSRenderer) TMSRenderer.getTMSRenderer(props,
					layerName);
			String[] layers = props[14].split("\\|");

			WMSRenderer wmsR = new WMSRenderer(new String[] { w.getURL() },
					w.getNAME(), w.getIMAGE_FILENAMEENDING(),
					w.getZOOM_MAXLEVEL(), w.getZoomMinLevel(),
					w.getMAPTILE_SIZEPX(), layers, w.getExtent(), w.getSRS(),
					props[15], null);

			try {
				wmsR.setInfo_format(props[16]);
			} catch (Exception ignore) {
				
			}
			
			return wmsR;
		} catch (Exception e) {
			logger.log(Level.SEVERE, "getWMSRenderer: ", e);
			return null;
		}
	}

	public static WMSRenderer getWMSRenderer(String[] baseUrl, String name,
			String imageFilenameEnding, int zoomMax, int zoomMin,
			int tileSizePX, String[] layers, Extent bbox, String srs,
			String version, Extent offExtent) {
		try {
			WMSRenderer r = new WMSRenderer(baseUrl, name, imageFilenameEnding,
					zoomMax, zoomMin, tileSizePX, layers, bbox, srs, version,
					offExtent);
			return r;
		} catch (Exception e) {
			logger.log(Level.SEVERE, "getWMSRenderer: ", e);
			return null;
		}
	}

	private void calculateResolutions(double size, int zoomMax) {
		try {
			resolutions = new double[zoomMax + 1];
			double iniRes = size / this.getMAPTILE_SIZEPX();
			for (int i = 0; i <= zoomMax; i++) {
				resolutions[i] = iniRes;
				size /= 2;
				iniRes = size / this.getMAPTILE_SIZEPX();
			}
		} catch (Exception e) {
			logger.log(Level.SEVERE, "calculateResolutions: ", e);
		}
	}

	@Override
	public String getTileURLString(int[] tileID, int zoomLevel) {
		try {
			// logger.debug("tile: " + tileID[0] + ", " + tileID[1]);
			final WMSStatus status = new WMSStatus();
			if (layers.length > 1) {
				final int length = layers.length;
				for (int i = 0; i < length; i++) {
					status.addLayerName(layers[i]);
				}
			} else {
				status.addLayerName(layers[0]);
			}

			status.setSrs(this.SRS);
			status.setFormat(this.getIMAGE_FILENAMEENDING());

			// if (CRSFactory.getCRS(this.SRS).getUnitsAbbrev().compareTo("g")
			// == 0) {
			// BBOX = TileConversor.tileBoundsG(tileID[1], tileID[0],
			// resolutions[zoomLevel]);
			// } else {

			Extent BBOX = TileConversor.tileMeterBounds(tileID[1], tileID[0],
					resolutions[zoomLevel]);
			// }

			status.setExtent(BBOX);
			status.setWidth(this.getMAPTILE_SIZEPX());
			status.setHeight(this.getMAPTILE_SIZEPX());

			final StringBuffer req = new StringBuffer();
			status.setDisconnectedServerURL(this.getBASEURL());
			// String info = FMapWMSDriverFactory.getFMapDriverForURL(
			// new URL(this.getBASEURL())).getFeatureInfo(status, 0, 0,
			// 10, this.neverCanceled);
			req.append(this.getBASEURL());
			String symbol = "?";
			if (this.getBASEURL().charAt(this.getBASEURL().length() - 1) == '?') {
				symbol = "";
			}

			req.append(symbol).append("REQUEST=GetMap&SERVICE=WMS&VERSION=")
					.append(version).append("&");
			req.append(getPartialQuery(status));
//			System.out.println(req.toString().replaceAll(" ", "%20"));
			return req.toString().replaceAll(" ", "%20");

		} catch (Exception e) {
			logger.log(Level.SEVERE, "getTileURLString: ", e);
			return null;
		}
	}

	protected String getPartialQuery(WMSStatus status) {
		final String fmt = status.getFormat();
		final StringBuffer req = new StringBuffer();
		String srs = "SRS";
		if (this.version.compareTo("1.3.0") == 0)
			srs = "CRS";
		req.append("LAYERS=")
				.append(Utilities.Vector2CS(status.getLayerNames()))
				.append("&").append(srs).append("=").append(status.getSrs())
				.append("&BBOX=").append(status.getExtent().getMinX())
				.append(",").append(status.getExtent().getMinY()).append(",")
				.append(status.getExtent().getMaxX()).append(",")
				.append(status.getExtent().getMaxY()).append("&WIDTH=")
				.append(status.getWidth()).append("&HEIGHT=")
				.append(status.getHeight()).append("&FORMAT=").append(fmt)
				.append("&STYLES=");

		Vector v = status.getStyles();
		if (v != null && v.size() > 0)
			req.append(Utilities.Vector2CS(v));
		v = status.getDimensions();
		if (v != null && v.size() > 0)
			req.append("&" + Utilities.Vector2URLParamString(v));
		if (status.getTransparency()) {
			req.append("&TRANSPARENT=TRUE");
		}
		return req.toString();
	}

	@Override
	public int isTMS() {
		return -1;
	}

	@Override
	public String toString() {
		StringBuffer layer = new StringBuffer();
		try {
			String base = super.toString();

			final int length = layers.length;
			StringBuffer layerNames = new StringBuffer();
			for (int i = 0; i < length; i++) {
				layerNames.append(layers[i]);
				if (i != length - 1) {
					layerNames.append("|");
				}
			}

			layer.append(base).append(",").append(layerNames);
			layer.append(",").append(version);
			layer.append(",").append(this.info_format);
			return layer.toString();

		} catch (Exception e) {
			logger.log(Level.SEVERE, "toString: ", e);
			return null;
		}
	}

	/**
	 * Instantiates a WMSStatus filling the properties: layerNames, SRS, format,
	 * width, height, disconnectedServerURL. The bounding box has to be filled
	 * separatelly
	 * 
	 * @return The WMSStatus with the current parameters of the WMSRenderer
	 */
	public WMSStatus buildWMSStatus() {
		WMSStatus status = null;
		try {
			status = new WMSStatus();
			if (layers.length > 1) {
				final int length = layers.length;
				for (int i = 0; i < length; i++) {
					status.addLayerName(layers[i]);
				}
			} else {
				status.addLayerName(layers[0]);
			}

			status.setSrs(this.SRS);
			status.setFormat(this.getIMAGE_FILENAMEENDING());
			status.setInfoFormat(this.getInfo_format());
			status.setWidth(this.getMAPTILE_SIZEPX());
			status.setHeight(this.getMAPTILE_SIZEPX());

			status.setDisconnectedServerURL(this.getBASEURL());
		} catch (Exception e) {
			logger.log(Level.SEVERE, "", e);
		}
		return status;
	}

	public String getInfo_format() {
		return info_format;
	}

	public void setInfo_format(String info_format) {
		this.info_format = info_format;
	}
}
